#ifndef bqtBindingQbHH
#define bqtBindingQbHH

#include <string>

#include "symtables.hh"
#include "lexer.hh" // for LexLocation
#include "stmt.hh"

/* Used by lexer */
void SetMetaDynamic();
void SetMetaStatic();

bool IsMetaDynamic();

Scope* GetRootScope();
Scope* GetCurrentScope();

std::string DeclareAsLabelInCurrentScope(const std::string& varname, bool is_definition);
bool AnchorLabelHere(const std::string& which);

struct VarContext
{
    // Pointer to the definition. nullptr = not defined.
    const VarContextDef* info;

    // True if it's imported using SHARED
    bool is_global;

    VarContext(): info(),is_global(false) {}

    operator bool() const { return info != nullptr; }
};

const UserType* FindUserType(const std::string& name);
bool DefineUserType(const std::string& name, UtypeP type);

expr* FindConstant(const std::string& name, bool duplicate_it);
int DefineConstant(const std::string& name, exprp&& p); // 1=ok, 0=dupdef, -1=typemismatch

// Check if the identifier has been defined (match_arrays if match arrays only, false if scalars only)
VarContext FindVariableFromScope(const std::string& name, bool match_arrays); // typesuffix optionally permitted

TypeP ImplicitlyDefineScalar(const std::string& name,
                             bool is_static=false,
                             bool is_shared=false,
                             TypeP type = nullptr,
                             LexLocation context = {}); // typesuffix optionally allowed
TypeP ImplicitlyDefineArray(const std::string& name,
                            unsigned num_dimensions,
                            long starting_index,
                            unsigned ending_index,
                            bool is_static=false,
                            bool is_shared=false,
                            TypeP type = nullptr,
                            LexLocation context = {}); // typesuffix optionally allowed

// The DIM statement.
const VarContextDef* ExplicitlyDefine_C(const LexLocation& context,
                                        const std::string& name, TypeP type,
                                        bool is_static,
                                        bool is_shared,
                                        bool was_implicit = false,
                                        int param_index = -1
                                       );
TypeP ExplicitlyDefine(const std::string& name, TypeP type,
                       bool is_static,
                       bool is_shared,
                       bool was_implicit,
                       LexLocation context = {});

// The same SUB/FUNCTION can have a DECLARE and a definition call.
const SubRoutine* DefineSubFunc(bool is_declare,
                                const std::string& name,
                                TypeP returntype,
                                const std::vector<SubRoutine::ParamSpec>& params,
                                bool is_static = false,
                                int callconvention = 0,
                                const std::string& linkername = ""
                               );
const SubRoutine* FindSubFunc(const std::string& name);

void EnterSubFuncScope(const std::string& name, bool everything_is_shared);
void ExitSubFuncScope(const std::string& name, stmtp&& code);
void SetCurrentScope(Scope* scope);
const std::string GetCurrentSubBaseName();

void SubScopeImportSharedVariable(const std::string& name, TypeP type, bool is_array);

void AnalyzeSubParamUsage();


/* Note: Array definition may fail if a SUB/FUNCTION exists by the same name.
 *       Scalar definition may fail if a CONST exists by the same name.
 *       Scalar definition may fail if a SUB/FUNCTION exists by the same name.
 *
 *       CONST definition may fail if a scalar exists by the same name.
 *       CONST definition may fail if a SUB/FUNCTION exists by the same name.
 *
 * Array definition will always check the $DYNAMIC state, but never allow redefinition.
 *
 */

#endif
