#ifndef bqtQBlibExecHH
#define bqtQBlibExecHH

#include "types.hh"
#include "error.hh"

extern "C" {

void B_StmtSTOP();

void B_LibInit();
void B_LibDone();

}

enum { VARPTR_TAG_BYTE=1, VARPTR_TAG_INT=2, VARPTR_TAG_STR=3, VARPTR_TAG_SGN=4, VARPTR_TAG_DBL=8, VARPTR_TAG_LNG=20,
       VARPTR_TAG_UBYTE=129, VARPTR_TAG_UINT=130, VARPTR_TAG_ULONG=148,
       VARPTR_LENGTH = (1 + (sizeof(unsigned long))) };
/* QB64 uses INT64=5, FLOAT=6, BIT=64+n , but for now we don't support them */

// VARPTR_Read*() are used by DRAW and PLAY.
template<typename CharP>
long double VARPTR_ReadNum(CharP s)
{
    unsigned long offs = 0;
    for(unsigned n=0; n<sizeof(offs); ++n)
    {
        offs |= ((unsigned long)((unsigned char)s[n+1])) << (n*8);
    }
    switch( (unsigned char) s[0])
    {
        case VARPTR_TAG_SGN: return *reinterpret_cast<const BAS_SGN*>(offs);
        case VARPTR_TAG_DBL: return *reinterpret_cast<const BAS_DBL*>(offs);

        case VARPTR_TAG_BYTE: return *reinterpret_cast<const BAS_BYTE*>(offs);
        case VARPTR_TAG_INT: return *reinterpret_cast<const BAS_INT*>(offs);
        case VARPTR_TAG_LNG: return *reinterpret_cast<const BAS_LNG*>(offs);

        case VARPTR_TAG_UBYTE: return *reinterpret_cast<const BAS_UBYTE*>(offs);
        case VARPTR_TAG_UINT: return *reinterpret_cast<const BAS_UINT*>(offs);
        case VARPTR_TAG_ULONG: return *reinterpret_cast<const BAS_ULONG*>(offs);

        default: B_ERROR(5);
    }
    return 0;
}

template<typename CharP>
const std::string& VARPTR_ReadStr(CharP s)
{
    unsigned long offs = 0;
    for(unsigned n=0; n<sizeof(offs); ++n)
    {
        offs |= ((unsigned long)((unsigned char)s[n+1])) << (n*8);
    }
    switch(s[0])
    {
        case VARPTR_TAG_STR: break;
        default: B_ERROR(5);
    }
    return *reinterpret_cast<const std::string*>(offs);
}

#endif
