#ifndef bqtQBlibTypesHH
#define bqtQBlibTypesHH

#include <stdint.h>
#include <type_traits>
#include <limits>
#include <string>

typedef int_least8_t   BAS_BYTE;
typedef uint_least8_t  BAS_UBYTE;
typedef int_least16_t  BAS_INT;
typedef int_least32_t  BAS_LNG;
typedef uint_least16_t BAS_UINT;
typedef uint_least32_t BAS_ULONG;

typedef float BAS_SGN;
typedef double BAS_DBL;

#include "bittypes.hh"

template<typename T> struct B_IsUnsignedType: public std::is_unsigned<T> {};
template<> struct B_IsUnsignedType<BAS_UBYTE>: public std::true_type {};
template<> struct B_IsUnsignedType<BAS_UINT>:  public std::true_type {};
template<> struct B_IsUnsignedType<BAS_ULONG>: public std::true_type {};

template<typename T> struct B_IsFloatType: public std::is_floating_point<T> {};
template<> struct B_IsFloatType<BAS_SGN>:  public std::true_type {};
template<> struct B_IsFloatType<BAS_DBL>: public std::true_type {};
template<> struct B_IsFloatType<long double>: public std::true_type {};

template<typename T> struct B_MinValue : public std::integral_constant<unsigned long long, std::numeric_limits<T>::min()> {};
template<typename T> struct B_MaxValue : public std::integral_constant<unsigned long long, std::numeric_limits<T>::max()> {};
template<> struct B_MinValue<BAS_SGN> : public std::integral_constant<unsigned long long, 0> {};
template<> struct B_MinValue<BAS_DBL> : public std::integral_constant<unsigned long long, 0> {};
template<> struct B_MinValue<long double> : public std::integral_constant<unsigned long long, 0> {};
template<> struct B_MaxValue<BAS_SGN> : public std::integral_constant<unsigned long long, 0> {};
template<> struct B_MaxValue<BAS_DBL> : public std::integral_constant<unsigned long long, 0> {};
template<> struct B_MaxValue<long double> : public std::integral_constant<unsigned long long, 0> {};

template<> struct B_MinValue<BAS_BYTE> : public std::integral_constant<unsigned long long, B_MaxValue<BAS_UBYTE>::value/2ul+1l> {};
template<> struct B_MinValue<BAS_INT>  : public std::integral_constant<unsigned long long, B_MaxValue<BAS_UINT> ::value/2ul+1l> {};
template<> struct B_MinValue<BAS_LNG>  : public std::integral_constant<unsigned long long, B_MaxValue<BAS_ULONG>::value/2ul+1l> {};

#endif
